{====================================================================}
{   TOxygenWndTransparency Component, v1.1  2000 Oxygen Software    }
{--------------------------------------------------------------------}
{     NB! This component works only under Microsoft Windows 2000     }
{--------------------------------------------------------------------}
{          Written by Oleg Fyodorov, ofyodorov@mtu-net.ru            }
{                    delphi@oxygensoftware.com                       }
{                  http://www.oxygensoftware.com                     }
{====================================================================}

{.$DEFINE O2_SW}

unit O2WndTransp;

  interface

    uses Windows, Classes, SysUtils, Graphics, Forms, Messages, Dialogs, Controls, ShellApi;

    type

      TOxygenWndTransparency = class(TComponent)
        private
          FOwnerWnd : HWnd;
          FWindowHandle : HWnd;
          FTransparencyFactor : Byte;
          FOnChangeTransparencyFactor : TNotifyEvent;
          procedure WndProc(var Msg: TMessage);
          procedure SetTransparencyFactor(const Value : Byte);
        protected
          procedure DoChangeTransparencyFactor; virtual;
        public
          constructor Create(AOwner : TComponent); override;
          destructor Destroy; override;
          procedure Show(Delay : Cardinal);
          procedure Hide(Delay : Cardinal);
{$IFNDEF O2_SW}
          {$IFNDEF VER100}
          function SetWndTransparency(const WindowTitle : String; const TransparencyValue : Byte) : Boolean; overload;
          function SetWndTransparency(const WindowHandle : Integer; const TransparencyValue : Byte) : Boolean; overload;
          function ResetWndTransparency(const WindowTitle : String) : Boolean; overload;
          function ResetWndTransparency(const WindowHandle : Integer) : Boolean; overload;
          {$ELSE}
          function SetWndTransparency(const WindowTitle : String; const TransparencyValue : Byte) : Boolean;
          function SetWndTransparencyHandle(const WindowHandle : Integer; const TransparencyValue : Byte) : Boolean;
          function ResetWndTransparency(const WindowTitle : String) : Boolean;
          function ResetWndTransparencyHandle(const WindowHandle : Integer) : Boolean;
          {$ENDIF}
{$ENDIF}
        published
          property TransparencyFactor : Byte read FTransparencyFactor write SetTransparencyFactor;
          property OnChangeTransparencyFactor : TNotifyEvent read FOnChangeTransparencyFactor write FOnChangeTransparencyFactor;
      end;

    procedure Register;

implementation

  const
      WS_EX_LAYERED = $80000;
      LWA_COLORKEY  = 1;
      LWA_ALPHA     = 2;

  function SetLayeredWindowAttributes(
            hwnd : HWND;         // handle to the layered window
            crKey : TColor;      // specifies the color key
            bAlpha : Byte;       // value for the blend function
            dwFlags : DWORD      // action
            ): BOOL; stdcall; external 'user32.dll';

// TOxygenWndTransparency
constructor TOxygenWndTransparency.Create(AOwner : TComponent);
begin
  if not (AOwner is TCustomForm) then raise Exception.Create('This component works only with TCustomForm descendants!');
  inherited Create(AOwner);
  FOnChangeTransparencyFactor := nil;
  FTransparencyFactor := 255;
  FOwnerWnd := (AOwner as TCustomForm).Handle;
  FWindowHandle := AllocateHWnd(WndProc);
{$IFDEF O2_SW}
  if (MessageDlg('This version of TOxygenWndTransparency is NOT REGISTERED. '+#13#10+
                 'Press Ok to visit http://www.oxygensoftware.com and register.',
                 mtWarning,[mbOk,mbCancel],0) = mrOk) then ShellExecute(0,'open','http://www.oxygensoftware.com',nil,nil,SW_SHOWNORMAL);
{$ENDIF}
end;

destructor TOxygenWndTransparency.Destroy;
begin
  DeallocateHWnd(FWindowHandle);
  inherited Destroy;
end;

procedure TOxygenWndTransparency.WndProc(var Msg: TMessage);
begin
  with Msg do
    if Msg = WM_CREATE then
      try
        SetTransparencyFactor(FTransparencyFactor);
      except
        Application.HandleException(Self);
      end
    else Result := DefWindowProc(FWindowHandle, Msg, wParam, lParam);
end;

procedure TOxygenWndTransparency.SetTransparencyFactor(const Value : Byte);
  var wl : DWord;
begin
  FTransparencyFactor := Value;
  if (csDesigning in ComponentState) then Exit;

  wl := GetWindowLong(FOwnerWnd, GWL_EXSTYLE);
  if ((wl and WS_EX_LAYERED) = 0) then
    if (SetWindowLong(FOwnerWnd, GWL_EXSTYLE, wl or WS_EX_LAYERED) = 0) then Exception.Create(SysErrorMessage(GetLastError));
  if not SetLayeredWindowAttributes(FOwnerWnd, 0, FTransparencyFactor, LWA_ALPHA) then Exception.Create(SysErrorMessage(GetLastError));
  DoChangeTransparencyFactor;
end;

procedure TOxygenWndTransparency.DoChangeTransparencyFactor;
begin
  if Assigned(FOnChangeTransparencyFactor) then FOnChangeTransparencyFactor(Self);
end;

procedure TOxygenWndTransparency.Show(Delay : Cardinal);
  var i : Byte;
begin
  for i := 0 to 255 do begin
    SetTransparencyFactor(i);
    Sleep(Delay);
  end;
end;

procedure TOxygenWndTransparency.Hide(Delay : Cardinal);
  var i : Byte;
begin
  for i := FTransparencyFactor downto 0 do begin
    SetTransparencyFactor(i);
    Sleep(Delay);
  end;
end;


function TOxygenWndTransparency.SetWndTransparency(const WindowTitle : String; const TransparencyValue : Byte) : Boolean;
  var wl : DWord;
      Wnd : HWnd;
begin
  Result := False;
  Wnd := FindWindow(nil,PChar(WindowTitle));
  if (Wnd = 0) then Exit;
  wl := GetWindowLong(Wnd, GWL_EXSTYLE);
  if ((wl and WS_EX_LAYERED) = 0) then if (SetWindowLong(Wnd, GWL_EXSTYLE, wl or WS_EX_LAYERED) = 0) then Exit;
  Result := SetLayeredWindowAttributes(Wnd, 0, TransparencyValue, LWA_ALPHA);
end;

function TOxygenWndTransparency.ResetWndTransparency(const WindowTitle : String) : Boolean;
  var wl : DWord;
      Wnd : HWnd;
begin
  Result := False;
  Wnd := FindWindow(nil,PChar(WindowTitle));
  if (Wnd = 0) then Exit;
  wl := (GetWindowLong(Wnd, GWL_EXSTYLE) and (not WS_EX_LAYERED));
  Result := (SetWindowLong(Wnd, GWL_EXSTYLE, wl) <> 0);
end;

{$IFNDEF VER100}
function TOxygenWndTransparency.SetWndTransparency(const WindowHandle : Integer; const TransparencyValue : Byte) : Boolean;
  var wl : DWord;
begin
  Result := False;
  if (WindowHandle = 0) then Exit;
  wl := GetWindowLong(WindowHandle, GWL_EXSTYLE);
  if ((wl and WS_EX_LAYERED) = 0) then if (SetWindowLong(WindowHandle, GWL_EXSTYLE, wl or WS_EX_LAYERED) = 0) then Exit;
  Result := SetLayeredWindowAttributes(WindowHandle, 0, TransparencyValue, LWA_ALPHA);
end;

function TOxygenWndTransparency.ResetWndTransparency(const WindowHandle : Integer) : Boolean;
  var wl : DWord;
begin
  Result := False;
  if (WindowHandle = 0) then Exit;
  wl := (GetWindowLong(WindowHandle, GWL_EXSTYLE) and (not WS_EX_LAYERED));
  Result := (SetWindowLong(WindowHandle, GWL_EXSTYLE, wl) <> 0);
end;

{$ELSE}

function TOxygenWndTransparency.SetWndTransparencyHandle(const WindowHandle : Integer; const TransparencyValue : Byte) : Boolean;
  var wl : DWord;
begin
  Result := False;
  if (WindowHandle = 0) then Exit;
  wl := GetWindowLong(WindowHandle, GWL_EXSTYLE);
  if ((wl and WS_EX_LAYERED) = 0) then if (SetWindowLong(WindowHandle, GWL_EXSTYLE, wl or WS_EX_LAYERED) = 0) then Exit;
  Result := SetLayeredWindowAttributes(WindowHandle, 0, TransparencyValue, LWA_ALPHA);
end;

function TOxygenWndTransparency.ResetWndTransparencyHandle(const WindowHandle : Integer) : Boolean;
  var wl : DWord;
begin
  Result := False;
  if (WindowHandle = 0) then Exit;
  wl := (GetWindowLong(WindowHandle, GWL_EXSTYLE) and (not WS_EX_LAYERED));
  Result := (SetWindowLong(WindowHandle, GWL_EXSTYLE, wl) <> 0);
end;

{$ENDIF}


procedure Register;
begin
  RegisterComponents('Oxygen', [TOxygenWndTransparency]);
end;

end.